@@ -1,11 +1,14 @@
 import re
 import logging
 
-from PyQt5.QtCore import Qt, QObject, pyqtSignal, QCoreApplication
-from PyQt5.QtWidgets import QApplication, QDialog, QVBoxLayout
+import pypeg2
+from PyQt5.QtCore import Qt, QObject, pyqtSignal, QCoreApplication, QLocale, QDateTime
+from PyQt5.QtWidgets import QApplication, QDialog, QVBoxLayout, QMessageBox
 
 from duniterpy.constants import PUBKEY_REGEX
 from duniterpy.documents import CRCPubkey
+from duniterpy.grammars.output import Condition
+
 from sakia.data.processors import ConnectionsProcessor
 from sakia.decorators import asyncify
 from sakia.gui.sub.password_input import PasswordInputController
@@ -13,6 +16,7 @@ from sakia.gui.sub.search_user.controller import SearchUserController
 from sakia.gui.sub.user_information.controller import UserInformationController
 from sakia.money import Quantitative
 from sakia.gui.widgets.dialogs import dialog_async_exec
+from sakia.services import sources
 from .model import TransferModel
 from .view import TransferView
 
@@ -34,7 +38,7 @@ class TransferController(QObject):
         """
         super().__init__()
         self.view = view
-        self.model = model
+        self.model = model  # type: TransferModel
         self.search_user = search_user
         self.user_information = user_information
         self.password_input = password_input
@@ -49,6 +53,7 @@ class TransferController(QObject):
         )
         self.view.spinbox_amount.valueChanged.connect(self.handle_amount_change)
         self.view.spinbox_relative.valueChanged.connect(self.handle_relative_change)
+        self.view.button_source_check.clicked.connect(self.check_source_dialog)
 
     @classmethod
     def create(cls, parent, app):
@@ -70,51 +75,73 @@ class TransferController(QObject):
             password_input.view,
         )
         model = TransferModel(app)
-        transfer = cls(view, model, search_user, user_information, password_input)
+        controller = cls(view, model, search_user, user_information, password_input)
 
         search_user.identity_selected.connect(user_information.search_identity)
 
-        view.set_keys(transfer.model.available_connections())
-        view.set_contacts(transfer.model.contacts())
+        view.set_keys(controller.model.available_connections())
+        view.set_contacts(controller.model.contacts())
         app.new_connection.connect(view.add_key)
         app.connection_removed.connect(view.remove_key)
 
-        return transfer
+        return controller
 
     @classmethod
     def integrate_to_main_view(cls, parent, app, connection):
-        transfer = cls.create(parent, app)
-        transfer.view.combo_connections.setCurrentText(connection.title())
-        transfer.view.radio_pubkey.toggle()
-        transfer.view.groupbox_connection.hide()
-        transfer.view.label_total.hide()
-        return transfer
+        controller = cls.create(parent, app)
+        controller.view.combo_connections.setCurrentText(connection.title())
+        controller.view.radio_pubkey.toggle()
+        controller.view.widget_connections.hide()
+        controller.view.widget_source.hide()
+        controller.view.label_total.hide()
+        return controller
 
     @classmethod
-    def open_transfer_with_pubkey(cls, parent, app, connection, pubkey):
-        transfer = cls.create(parent, app)
-        transfer.view.groupbox_connection.show()
+    def open_transfer_with_pubkey(cls, parent, app, connection, pubkey, source):
+        controller = cls.create(parent, app)
+        controller.view.widget_connections.show()
         if connection:
-            transfer.view.combo_connections.setCurrentText(connection.title())
-        transfer.view.edit_pubkey.setText(pubkey)
-        transfer.view.radio_pubkey.setChecked(True)
-        transfer.view.radio_pubkey.toggle()
+            controller.view.combo_connections.setCurrentText(connection.title())
+        if pubkey:
+            controller.view.edit_pubkey.setText(pubkey)
+            controller.view.radio_pubkey.setChecked(True)
+            controller.view.radio_pubkey.toggle()
+        else:
+            controller.view.radio_local_key.setChecked(True)
+            controller.view.radio_local_key.toggle()
+        if source:
+            controller.model.current_source = source
+            controller.view.label_source_identifier.setText(
+                "{}:{}".format(source.identifier, source.noffset)
+            )
+            controller.set_amount_value(source.amount, source.base)
+            controller.view.spinbox_amount.setDisabled(True)
+            controller.view.spinbox_relative.setDisabled(True)
+            result, _ = controller.check_source(source)
+            # by default, source is unlocked, if not...
+            if not result:
+                # enabled the check button to open the errors dialog
+                controller.view.button_source_check.setEnabled(True)
+        else:
+            controller.view.widget_source.hide()
 
-        transfer.refresh()
-        return transfer
+        controller.refresh()
+        return controller
 
     @classmethod
-    def send_money_to_pubkey(cls, parent, app, connection, pubkey):
+    def send_money_to_pubkey(cls, parent, app, connection, pubkey, source):
         dialog = QDialog(parent)
         dialog.setWindowTitle(
             QCoreApplication.translate("TransferController", "Transfer")
         )
         dialog.setLayout(QVBoxLayout(dialog))
-        transfer = cls.open_transfer_with_pubkey(parent, app, connection, pubkey)
+        controller = cls.open_transfer_with_pubkey(
+            parent, app, connection, pubkey, source
+        )
 
-        dialog.layout().addWidget(transfer.view)
-        transfer.accepted.connect(dialog.accept)
-        transfer.rejected.connect(dialog.reject)
+        dialog.layout().addWidget(controller.view)
+        controller.accepted.connect(dialog.accept)
+        controller.rejected.connect(dialog.reject)
         return dialog.exec()
 
     @classmethod
@@ -124,14 +151,14 @@ class TransferController(QObject):
             QCoreApplication.translate("TransferController", "Transfer")
         )
         dialog.setLayout(QVBoxLayout(dialog))
-        transfer = cls.open_transfer_with_pubkey(
-            parent, app, connection, identity.pubkey
+        controller = cls.open_transfer_with_pubkey(
+            parent, app, connection, identity.pubkey, None
         )
 
-        transfer.user_information.change_identity(identity)
-        dialog.layout().addWidget(transfer.view)
-        transfer.accepted.connect(dialog.accept)
-        transfer.rejected.connect(dialog.reject)
+        controller.user_information.change_identity(identity)
+        dialog.layout().addWidget(controller.view)
+        controller.accepted.connect(dialog.accept)
+        controller.rejected.connect(dialog.reject)
         return dialog.exec()
 
     @classmethod
@@ -141,34 +168,27 @@ class TransferController(QObject):
             QCoreApplication.translate("TransferController", "Transfer")
         )
         dialog.setLayout(QVBoxLayout(dialog))
-        transfer = cls.create(parent, app)
-        transfer.view.groupbox_connection.show()
-        transfer.view.label_total.show()
-        transfer.view.combo_connections.setCurrentText(connection.title())
-        transfer.view.edit_pubkey.setText(resent_transfer.receivers[0])
-        transfer.view.radio_pubkey.setChecked(True)
+        controller = cls.create(parent, app)
+        controller.view.widget_connections.show()
+        controller.view.label_total.show()
+        controller.view.combo_connections.setCurrentText(connection.title())
+        controller.view.edit_pubkey.setText(resent_transfer.receivers[0])
+        controller.view.radio_pubkey.setChecked(True)
 
-        transfer.refresh()
+        controller.refresh()
 
-        current_base = transfer.model.current_base()
-        current_base_amount = resent_transfer.amount / pow(
-            10, resent_transfer.amount_base - current_base
-        )
-
-        relative = transfer.model.quant_to_rel(current_base_amount / 100)
-        transfer.view.set_spinboxes_parameters(current_base_amount / 100, relative)
-        transfer.view.change_relative_amount(relative)
-        transfer.view.change_quantitative_amount(current_base_amount / 100)
+        # display transaction amount
+        controller.set_amount_value(resent_transfer.amount, resent_transfer.amount_base)
 
         connections_processor = ConnectionsProcessor.instanciate(app)
         wallet_index = connections_processor.connections().index(connection)
-        transfer.view.combo_connections.setCurrentIndex(wallet_index)
-        transfer.view.edit_pubkey.setText(resent_transfer.receivers[0])
-        transfer.view.radio_pubkey.toggle()
-        transfer.view.edit_message.setText(resent_transfer.comment)
-        dialog.layout().addWidget(transfer.view)
-        transfer.accepted.connect(dialog.accept)
-        transfer.rejected.connect(dialog.reject)
+        controller.view.combo_connections.setCurrentIndex(wallet_index)
+        controller.view.edit_pubkey.setText(resent_transfer.receivers[0])
+        controller.view.radio_pubkey.toggle()
+        controller.view.edit_message.setText(resent_transfer.comment)
+        dialog.layout().addWidget(controller.view)
+        controller.accepted.connect(dialog.accept)
+        controller.rejected.connect(dialog.reject)
         return dialog.exec()
 
     def valid_crc_pubkey(self):
@@ -215,7 +235,8 @@ class TransferController(QObject):
     async def accept(self):
         logging.debug("Accept transfer action...")
         self.view.button_box.setEnabled(False)
-        comment = self.view.edit_message.text()
+
+        source = self.model.current_source
 
         logging.debug("checking recipient mode...")
         recipient = self.selected_pubkey()
@@ -228,9 +249,19 @@ class TransferController(QObject):
         logging.debug("Setting cursor...")
         QApplication.setOverrideCursor(Qt.WaitCursor)
 
+        comment = self.view.edit_message.text()
+        lock_mode = self.view.combo_locks.currentIndex()
+
         logging.debug("Send money...")
         result, transactions = await self.model.send_money(
-            recipient, secret_key, password, amount, amount_base, comment
+            recipient,
+            secret_key,
+            password,
+            amount,
+            amount_base,
+            comment,
+            lock_mode,
+            source,
         )
         if result[0]:
             await self.view.show_success(self.model.notifications(), recipient)
@@ -271,9 +302,13 @@ class TransferController(QObject):
             else:
                 self.view.set_button_box(TransferView.ButtonBoxState.NO_RECEIVER)
         elif self.password_input.valid():
+            # if source and check source successful...
             self.view.set_button_box(TransferView.ButtonBoxState.OK)
         else:
             self.view.set_button_box(TransferView.ButtonBoxState.WRONG_PASSWORD)
+        # if source and check source button still enabled...
+        if self.model.current_source and self.view.button_source_check.isEnabled():
+            self.view.set_button_box(TransferView.ButtonBoxState.SOURCE_LOCKED)
 
         max_relative = self.model.quant_to_rel(current_base_amount / 100)
         self.view.spinbox_amount.setSuffix(Quantitative.base_str(current_base))
@@ -294,3 +329,95 @@ class TransferController(QObject):
         self.model.set_connection(index)
         self.password_input.set_connection(self.model.connection)
         self.refresh()
+
+    def check_source(self, source):
+        """
+        Check source conditions lock status
+
+        Return a tupple with :
+            result: bool,
+            errors: list of tuples
+                [(condition: str, message: str, info: int|str),...]
+
+        :param source:
+        :return tuple:
+        """
+        # evaluate condition
+        condition = pypeg2.parse(source.conditions, Condition)
+        result, _errors = self.model.app.sources_service.evaluate_condition(
+            self.model.app.currency,
+            condition,
+            [self.model.connection.pubkey],
+            [],
+            source.identifier,
+        )
+        return result, _errors
+
+    def check_source_dialog(self):
+        """
+        Open check source result and errors dialog
+
+        :return:
+        """
+        source = self.model.current_source
+        result, _errors = self.check_source(source)
+
+        # if success...
+        if result:
+            message = QCoreApplication.translate(
+                "TransferController", "Check is successful!"
+            )
+            self.view.button_source_check.setDisabled(True)
+            self.refresh()
+        # if failure...
+        else:
+            message = QCoreApplication.translate(
+                "TransferController", "<p><b>Condition</b></p>{}"
+            ).format(source.conditions)
+            message += QCoreApplication.translate(
+                "TransferController", "<p><b>Errors</b><p>"
+            )
+            message += "<ul>"
+            # add error messages
+            for (_condition, _error, _param) in _errors:
+                if isinstance(_param, int):
+                    _param = (
+                        QLocale.toString(
+                            QLocale(),
+                            QDateTime.fromTime_t(_param),
+                            QLocale.dateTimeFormat(QLocale(), QLocale.ShortFormat),
+                        )
+                        + " BAT"
+                    )
+                message += QCoreApplication.translate(
+                    "TransferController",
+                    '<li>Error in {}: <span style="color: red">{} {}</span></li>',
+                ).format(
+                    _condition,
+                    QCoreApplication.translate("SourcesServices", _error),
+                    _param,
+                )
+            message += "</ul>"
+        # open message box displaying source check result
+        qmessagebox = QMessageBox(self.view)
+        qmessagebox.setWindowTitle(
+            QCoreApplication.translate("TransferController", "Check source condition")
+        )
+        qmessagebox.setText(message)
+        qmessagebox.exec()
+
+    def set_amount_value(self, amount, base):
+        """
+        Set quantitative and relative amounts from amount and base given
+        :param int amount: Amount to display
+        :param int base: Base of the amount given
+        :return:
+        """
+        # calculate amount for current base
+        current_base = self.model.current_base()
+        current_base_amount = amount / pow(10, base - current_base)
+        # display quantitative and relative amounts
+        relative = self.model.quant_to_rel(current_base_amount / 100)
+        self.view.set_spinboxes_parameters(current_base_amount / 100, relative)
+        self.view.change_relative_amount(relative)
+        self.view.change_quantitative_amount(current_base_amount / 100)
