@@ -2,6 +2,7 @@ import jsonschema
 import attr
 import logging
 
+import pypeg2
 from duniterpy.key import SigningKey
 from duniterpy.documents import (
     Certification,
@@ -12,12 +13,10 @@ from duniterpy.documents import (
     SIGParameter,
     Unlock,
     block_uid,
-    BlockUID,
 )
-from duniterpy.documents import Identity as IdentityDoc
 from duniterpy.documents import Transaction as TransactionDoc
 from duniterpy.documents.transaction import reduce_base
-from duniterpy.grammars import output
+from duniterpy.grammars.output import Condition, Operator, SIG, CSV
 from duniterpy.api import bma
 from sakia.data.entities import Identity, Transaction, Source
 from sakia.data.processors import (
@@ -27,9 +26,11 @@ from sakia.data.processors import (
     TransactionsProcessor,
     SourcesProcessor,
     CertificationsProcessor,
+    ConnectionsProcessor,
 )
 from sakia.data.connectors import BmaConnector, parse_bma_responses
 from sakia.errors import NotEnoughChangeError
+from sakia.services.sources import SourcesServices
 
 
 @attr.s()
@@ -51,6 +52,7 @@ class DocumentsService:
     _certifications_processor = attr.ib()
     _transactions_processor = attr.ib()
     _sources_processor = attr.ib()
+    _sources_services = attr.ib()  # type: SourcesServices
     _logger = attr.ib(default=attr.Factory(lambda: logging.getLogger("sakia")))
 
     @classmethod
@@ -66,6 +68,14 @@ class DocumentsService:
             CertificationsProcessor.instanciate(app),
             TransactionsProcessor.instanciate(app),
             SourcesProcessor.instanciate(app),
+            SourcesServices(
+                app.currency,
+                SourcesProcessor.instanciate(app),
+                ConnectionsProcessor.instanciate(app),
+                TransactionsProcessor.instanciate(app),
+                BlockchainProcessor.instanciate(app),
+                BmaConnector(NodesProcessor(app.db.nodes_repo), app.parameters),
+            ),
         )
 
     def generate_identity(self, connection):
@@ -286,13 +296,13 @@ class DocumentsService:
 
         return document.signed_raw(), identity
 
-    def tx_sources(self, amount, amount_base, currency, pubkey):
+    def tx_sources(self, amount, amount_base, currency, key: SigningKey):
         """
         Get inputs to generate a transaction with a given amount of money
         :param int amount: The amount target value
         :param int amount_base: The amount base target value
         :param str currency: The community target of the transaction
-        :param str pubkey: The pubkey owning the sources
+        :param SigningKey key: The key owning the sources
         :return: The list of inputs to use in the transaction document
         """
 
@@ -309,7 +319,7 @@ class DocumentsService:
             return i
 
         amount, amount_base = reduce_base(amount, amount_base)
-        available_sources = self._sources_processor.available(currency, pubkey)
+        available_sources = self._sources_processor.available(currency, key.pubkey)
         if available_sources:
             current_base = max([src.base for src in available_sources])
             value = 0
@@ -319,6 +329,13 @@ class DocumentsService:
             buf_sources = list(available_sources)
             while current_base >= 0:
                 for s in [src for src in available_sources if src.base == current_base]:
+                    condition = pypeg2.parse(s.conditions, Condition)
+                    # evaluate the condition
+                    result, _ = self._sources_services.evaluate_condition(
+                        currency, condition, [key.pubkey], [], s.identifier
+                    )
+                    if not result:
+                        continue
                     test_sources = sources + [s]
                     val = current_value(test_sources, overheads)
                     # if we have to compute an overhead
@@ -354,7 +371,10 @@ class DocumentsService:
                 current_base -= 1
 
         raise NotEnoughChangeError(
-            value, currency, len(sources), amount * pow(10, amount_base)
+            current_value(available_sources, []),
+            currency,
+            len(available_sources),
+            amount * pow(10, amount_base),
         )
 
     def tx_inputs(self, sources):
@@ -381,9 +401,10 @@ class DocumentsService:
             unlocks.append(Unlock(i, [SIGParameter(0)]))
         return unlocks
 
-    def tx_outputs(self, issuer, receiver, outputs, overheads):
+    def tx_outputs(self, issuer, receiver, outputs, overheads, lock_mode):
         """
         Get outputs to generate a transaction with a given amount of money
+        :param int lock_mode: Index of the selected spend condition
         :param str issuer: The issuer of the transaction
         :param str receiver: The target of the transaction
         :param list outputs: The amount to send
@@ -391,6 +412,22 @@ class DocumentsService:
         :param list overheads: The overheads used to send the given amount of money
         :return: The list of outputs to use in the transaction document
         """
+        lock_modes = {
+            # Receiver
+            0: pypeg2.compose(Condition.token(SIG.token(receiver)), Condition),
+            # Receiver or (issuer and delay of one week)
+            1: pypeg2.compose(
+                Condition.token(
+                    SIG.token(receiver),
+                    Operator.token("||"),
+                    Condition.token(
+                        SIG.token(issuer), Operator.token("&&"), CSV.token(604800),
+                    ),
+                ),
+                Condition,
+            ),
+        }
+
         total = []
         outputs_bases = set(o[1] for o in outputs)
         for base in outputs_bases:
@@ -400,15 +437,7 @@ class DocumentsService:
                     output_sum += o[0]
             # fixme: OutputSource condition argument should be an instance of Condition, not a string
             #        it is not to the user to construct the condition script, but to the dedicated classes
-            total.append(
-                OutputSource(
-                    output_sum,
-                    base,
-                    output.Condition.token(output.SIG.token(receiver)).compose(
-                        output.Condition()
-                    ),
-                )
-            )
+            total.append(OutputSource(output_sum, base, lock_modes[lock_mode]))
 
         overheads_bases = set(o[1] for o in overheads)
         for base in overheads_bases:
@@ -422,9 +451,7 @@ class DocumentsService:
                 OutputSource(
                     overheads_sum,
                     base,
-                    output.Condition.token(output.SIG.token(issuer)).compose(
-                        output.Condition()
-                    ),
+                    pypeg2.compose(Condition.token(SIG.token(issuer)), Condition),
                 )
             )
 
@@ -439,20 +466,32 @@ class DocumentsService:
         :return:
         """
         for offset, output in enumerate(txdoc.outputs):
-            if output.condition.left.pubkey == pubkey:
+            if self._sources_services.find_signature_in_condition(
+                output.condition, pubkey
+            ):
                 source = Source(
                     currency=currency,
                     pubkey=pubkey,
                     identifier=txdoc.sha_hash,
-                    type="T",
+                    type=Source.TYPE_TRANSACTION,
                     noffset=offset,
                     amount=output.amount,
                     base=output.base,
+                    conditions=pypeg2.compose(output.condition, Condition),
                 )
                 self._sources_processor.insert(source)
 
     def prepare_tx(
-        self, key, receiver, blockstamp, amount, amount_base, message, currency
+        self,
+        key,
+        receiver,
+        blockstamp,
+        amount,
+        amount_base,
+        message,
+        currency,
+        lock_mode=0,
+        source=None,
     ):
         """
         Prepare a simple Transaction document
@@ -463,38 +502,48 @@ class DocumentsService:
         :param int amount_base: the amount base of the currency
         :param str message: the comment of the tx
         :param str currency: the target community
+        :param int lock_mode: Lock condition mode selected in combo box
+        :param Source source: Source instance or None
         :return: the transaction document
         :rtype: List[sakia.data.entities.Transaction]
         """
         forged_tx = []
-        sources = [None] * 41
-        while len(sources) > 40:
-            result = self.tx_sources(int(amount), amount_base, currency, key.pubkey)
-            sources = result[0]
-            computed_outputs = result[1]
-            overheads = result[2]
-            # Fix issue #594
-            if len(sources) > 40:
-                sources_value = 0
-                for s in sources[:39]:
-                    sources_value += s.amount * (10 ** s.base)
-                sources_value, sources_base = reduce_base(sources_value, 0)
-                chained_tx = self.prepare_tx(
-                    key,
-                    key.pubkey,
-                    blockstamp,
-                    sources_value,
-                    sources_base,
-                    "[CHAINED]",
-                    currency,
-                )
-                forged_tx += chained_tx
-        self._sources_processor.consume(sources)
+        if source is None:
+            # automatic selection of sources
+            sources = [None] * 41
+            while len(sources) > 40:
+                result = self.tx_sources(int(amount), amount_base, currency, key)
+                sources = result[0]
+                computed_outputs = result[1]
+                overheads = result[2]
+                # Fix issue #594
+                if len(sources) > 40:
+                    sources_value = 0
+                    for s in sources[:39]:
+                        sources_value += s.amount * (10 ** s.base)
+                    sources_value, sources_base = reduce_base(sources_value, 0)
+                    chained_tx = self.prepare_tx(
+                        key,
+                        key.pubkey,
+                        blockstamp,
+                        sources_value,
+                        sources_base,
+                        "[CHAINED]",
+                        currency,
+                    )
+                    forged_tx += chained_tx
+        else:
+            sources = [source]
+            computed_outputs = [(source.amount, source.base)]
+            overheads = []
+
         logging.debug("Inputs: {0}".format(sources))
 
         inputs = self.tx_inputs(sources)
         unlocks = self.tx_unlocks(sources)
-        outputs = self.tx_outputs(key.pubkey, receiver, computed_outputs, overheads)
+        outputs = self.tx_outputs(
+            key.pubkey, receiver, computed_outputs, overheads, lock_mode
+        )
         logging.debug("Outputs: {0}".format(outputs))
         txdoc = TransactionDoc(
             10,
@@ -530,13 +579,26 @@ class DocumentsService:
             raw=txdoc.signed_raw(),
         )
         forged_tx.append(tx)
+
+        self._sources_processor.consume(sources, tx.sha_hash)
+
         return forged_tx
 
     async def send_money(
-        self, connection, secret_key, password, recipient, amount, amount_base, message
+        self,
+        connection,
+        secret_key,
+        password,
+        recipient,
+        amount,
+        amount_base,
+        message,
+        lock_mode,
+        source,
     ):
         """
         Send money to a given recipient in a specified community
+        :param int lock_mode: Index in the combo_locks combobox
         :param sakia.data.entities.Connection connection: The account salt
         :param str secret_key: The account secret_key
         :param str password: The account password
@@ -544,6 +606,7 @@ class DocumentsService:
         :param int amount: The amount of money to transfer
         :param int amount_base: The amount base of the transfer
         :param str message: The message to send with the transfer
+        :param Source source: Source instance or None
         """
         blockstamp = self._blockchain_processor.current_buid(connection.currency)
         key = SigningKey.from_credentials(
@@ -561,6 +624,8 @@ class DocumentsService:
                 amount_base,
                 message,
                 connection.currency,
+                lock_mode,
+                source,
             )
 
             for i, tx in enumerate(tx_entities):
